package sun.security.jgss;

import com.sun.security.jgss.InquireSecContextPermission;
import com.sun.security.jgss.InquireType;
import org.ietf.jgss.MessageProp;
import sun.security.jgss.spi.GSSCredentialSpi;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.ChannelBinding;
import sun.security.util.ObjectIdentifier;
import org.ietf.jgss.Oid;
import sun.security.jgss.spi.GSSContextSpi;
import com.sun.security.jgss.ExtendedGSSContext;

class GSSContextImpl implements ExtendedGSSContext
{
    private final GSSManagerImpl gssManager;
    private final boolean initiator;
    private static final int PRE_INIT = 1;
    private static final int IN_PROGRESS = 2;
    private static final int READY = 3;
    private static final int DELETED = 4;
    private int currentState;
    private GSSContextSpi mechCtxt;
    private Oid mechOid;
    private ObjectIdentifier objId;
    private GSSCredentialImpl myCred;
    private GSSNameImpl srcName;
    private GSSNameImpl targName;
    private int reqLifetime;
    private ChannelBinding channelBindings;
    private boolean reqConfState;
    private boolean reqIntegState;
    private boolean reqMutualAuthState;
    private boolean reqReplayDetState;
    private boolean reqSequenceDetState;
    private boolean reqCredDelegState;
    private boolean reqAnonState;
    private boolean reqDelegPolicyState;
    
    public GSSContextImpl(GSSManagerImpl gssManager, GSSName gssName, Oid default_MECH_OID, GSSCredential gssCredential, int reqLifetime) throws GSSException {
        this.currentState = 1;
        this.mechCtxt = null;
        this.mechOid = null;
        this.objId = null;
        this.myCred = null;
        this.srcName = null;
        this.targName = null;
        this.reqLifetime = Integer.MAX_VALUE;
        this.channelBindings = null;
        this.reqConfState = true;
        this.reqIntegState = true;
        this.reqMutualAuthState = true;
        this.reqReplayDetState = true;
        this.reqSequenceDetState = true;
        this.reqCredDelegState = false;
        this.reqAnonState = false;
        this.reqDelegPolicyState = false;
        if (gssName == null || !(gssName instanceof GSSNameImpl)) {
            throw new GSSException(3);
        }
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        this.gssManager = gssManager;
        this.myCred = (GSSCredentialImpl)gssCredential;
        this.reqLifetime = reqLifetime;
        this.targName = (GSSNameImpl)gssName;
        this.mechOid = default_MECH_OID;
        this.initiator = true;
        return;
    }
    
    public GSSContextImpl(GSSManagerImpl gssManager, GSSCredential gssCredential) throws GSSException {
        this.currentState = 1;
        this.mechCtxt = null;
        this.mechOid = null;
        this.objId = null;
        this.myCred = null;
        this.srcName = null;
        this.targName = null;
        this.reqLifetime = Integer.MAX_VALUE;
        this.channelBindings = null;
        this.reqConfState = true;
        this.reqIntegState = true;
        this.reqMutualAuthState = true;
        this.reqReplayDetState = true;
        this.reqSequenceDetState = true;
        this.reqCredDelegState = false;
        this.reqAnonState = false;
        this.reqDelegPolicyState = false;
        this.gssManager = gssManager;
        this.myCred = (GSSCredentialImpl)gssCredential;
        this.initiator = false;
    }
    
    public GSSContextImpl(GSSManagerImpl gssManager, byte[] array) throws GSSException {
        this.currentState = 1;
        this.mechCtxt = null;
        this.mechOid = null;
        this.objId = null;
        this.myCred = null;
        this.srcName = null;
        this.targName = null;
        this.reqLifetime = Integer.MAX_VALUE;
        this.channelBindings = null;
        this.reqConfState = true;
        this.reqIntegState = true;
        this.reqMutualAuthState = true;
        this.reqReplayDetState = true;
        this.reqSequenceDetState = true;
        this.reqCredDelegState = false;
        this.reqAnonState = false;
        this.reqDelegPolicyState = false;
        this.gssManager = gssManager;
        this.mechCtxt = gssManager.getMechanismContext(array);
        this.initiator = this.mechCtxt.isInitiator();
        this.mechOid = this.mechCtxt.getMech();
    }
    
    @Override
    public byte[] initSecContext(byte[] buf, int offset, int length) throws GSSException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(600);
        return (this.initSecContext(new ByteArrayInputStream(buf, offset, length), byteArrayOutputStream) == 0) ? null : byteArrayOutputStream.toByteArray();
    }
    
    @Override
    public int initSecContext(InputStream inputStream, OutputStream outputStream) throws GSSException {
        if (this.mechCtxt != null && this.currentState != 2) {
            throw new GSSExceptionImpl(11, "Illegal call to initSecContext");
        }
        int mechTokenLength = -1;
        GSSCredentialSpi gssCredentialSpi = null;
        boolean n = false;
        try {
            if (this.mechCtxt == null) {
                if (this.myCred != null) {
                    try {
                        gssCredentialSpi = this.myCred.getElement(this.mechOid, true);
                    }
                    catch (GSSException ex) {
                        if (GSSUtil.isSpNegoMech(this.mechOid) && ex.getMajor() == 13) {
                            gssCredentialSpi = this.myCred.getElement(this.myCred.getMechs()[0], true);
                        }
                        else {
                            throw ex;
                        }
                    }
                }
                (this.mechCtxt = this.gssManager.getMechanismContext(this.targName.getElement(this.mechOid), gssCredentialSpi, this.reqLifetime, this.mechOid)).requestConf(this.reqConfState);
                this.mechCtxt.requestInteg(this.reqIntegState);
                this.mechCtxt.requestCredDeleg(this.reqCredDelegState);
                this.mechCtxt.requestMutualAuth(this.reqMutualAuthState);
                this.mechCtxt.requestReplayDet(this.reqReplayDetState);
                this.mechCtxt.requestSequenceDet(this.reqSequenceDetState);
                this.mechCtxt.requestAnonymity(this.reqAnonState);
                this.mechCtxt.setChannelBinding(this.channelBindings);
                this.mechCtxt.requestDelegPolicy(this.reqDelegPolicyState);
                this.objId = new ObjectIdentifier(this.mechOid.toString());
                this.currentState = 2;
                n = true;
            }
            else if (!this.mechCtxt.getProvider().getName().equals("SunNativeGSS") && !GSSUtil.isSpNegoMech(this.mechOid)) {
                GSSHeader gssHeader = new GSSHeader(inputStream);
                if (!gssHeader.getOid().equals(this.objId)) {
                    throw new GSSExceptionImpl(10, "Mechanism not equal to " + this.mechOid.toString() + " in initSecContext token");
                }
                mechTokenLength = gssHeader.getMechTokenLength();
            }
            byte[] initSecContext = this.mechCtxt.initSecContext(inputStream, mechTokenLength);
            int length = 0;
            if (initSecContext != null) {
                length = initSecContext.length;
                if (!this.mechCtxt.getProvider().getName().equals("SunNativeGSS") && (n || !GSSUtil.isSpNegoMech(this.mechOid))) {
                    length += new GSSHeader(this.objId, initSecContext.length).encode(outputStream);
                }
                outputStream.write(initSecContext);
            }
            if (this.mechCtxt.isEstablished()) {
                this.currentState = 3;
            }
            return length;
        }
        catch (IOException ex2) {
            throw new GSSExceptionImpl(10, ex2.getMessage());
        }
    }
    
    @Override
    public byte[] acceptSecContext(byte[] buf, int offset, int length) throws GSSException {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(100);
        this.acceptSecContext(new ByteArrayInputStream(buf, offset, length), byteArrayOutputStream);
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        return (byteArray.length == 0) ? null : byteArray;
    }
    
    @Override
    public void acceptSecContext(InputStream inputStream, OutputStream outputStream) throws GSSException {
        if (this.mechCtxt != null && this.currentState != 2) {
            throw new GSSExceptionImpl(11, "Illegal call to acceptSecContext");
        }
        int n = -1;
        GSSCredentialSpi element = null;
        try {
            if (this.mechCtxt == null) {
                GSSHeader gssHeader2 = new GSSHeader(inputStream);
                n = gssHeader2.getMechTokenLength();
                this.objId = gssHeader2.getOid();
                this.mechOid = new Oid(this.objId.toString());
                if (this.myCred != null) {
                    element = this.myCred.getElement(this.mechOid, false);
                }
                (this.mechCtxt = this.gssManager.getMechanismContext(element, this.mechOid)).setChannelBinding(this.channelBindings);
                this.currentState = 2;
            }
            else if (!this.mechCtxt.getProvider().getName().equals("SunNativeGSS") && !GSSUtil.isSpNegoMech(this.mechOid)) {
                GSSHeader gssHeader = new GSSHeader(inputStream);
                if (!gssHeader.getOid().equals(this.objId)) {
                    throw new GSSExceptionImpl(10, "Mechanism not equal to " + this.mechOid.toString() + " in acceptSecContext token");
                }
                n = gssHeader.getMechTokenLength();
            }
            byte[] acceptSecContext = this.mechCtxt.acceptSecContext(inputStream, n);
            if (acceptSecContext != null) {
                int length = acceptSecContext.length;
                if (!this.mechCtxt.getProvider().getName().equals("SunNativeGSS") && !GSSUtil.isSpNegoMech(this.mechOid)) {
                    int n2 = length + new GSSHeader(this.objId, acceptSecContext.length).encode(outputStream);
                }
                outputStream.write(acceptSecContext);
            }
            if (this.mechCtxt.isEstablished()) {
                this.currentState = 3;
            }
        }
        catch (IOException ex) {
            throw new GSSExceptionImpl(10, ex.getMessage());
        }
        return;
    }
    
    @Override
    public boolean isEstablished() {
        return this.mechCtxt != null && this.currentState == 3;
    }
    
    @Override
    public int getWrapSizeLimit(int n, boolean b, int n2) throws GSSException {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getWrapSizeLimit(n, b, n2);
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public byte[] wrap(byte[] array, int n, int n2, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            return this.mechCtxt.wrap(array, n, n2, messageProp);
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public void wrap(InputStream inputStream, OutputStream outputStream, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            this.mechCtxt.wrap(inputStream, outputStream, messageProp);
            return;
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public byte[] unwrap(byte[] array, int n, int n2, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            return this.mechCtxt.unwrap(array, n, n2, messageProp);
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public void unwrap(InputStream inputStream, OutputStream outputStream, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            this.mechCtxt.unwrap(inputStream, outputStream, messageProp);
            return;
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public byte[] getMIC(byte[] array, int n, int n2, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getMIC(array, n, n2, messageProp);
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public void getMIC(InputStream inputStream, OutputStream outputStream, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            this.mechCtxt.getMIC(inputStream, outputStream, messageProp);
            return;
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public void verifyMIC(byte[] array, int n, int n2, byte[] array2, int n3, int n4, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            this.mechCtxt.verifyMIC(array, n, n2, array2, n3, n4, messageProp);
            return;
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public void verifyMIC(InputStream inputStream, InputStream inputStream2, MessageProp messageProp) throws GSSException {
        if (this.mechCtxt != null) {
            this.mechCtxt.verifyMIC(inputStream, inputStream2, messageProp);
            return;
        }
        throw new GSSExceptionImpl(12, "No mechanism context yet!");
    }
    
    @Override
    public byte[] export() throws GSSException {
        byte[] export = null;
        if (this.mechCtxt.isTransferable() && this.mechCtxt.getProvider().getName().equals("SunNativeGSS")) {
            export = this.mechCtxt.export();
            return export;
        }
        return export;
    }
    
    @Override
    public void requestMutualAuth(boolean reqMutualAuthState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqMutualAuthState = reqMutualAuthState;
        }
    }
    
    @Override
    public void requestReplayDet(boolean reqReplayDetState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqReplayDetState = reqReplayDetState;
        }
    }
    
    @Override
    public void requestSequenceDet(boolean reqSequenceDetState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqSequenceDetState = reqSequenceDetState;
        }
    }
    
    @Override
    public void requestCredDeleg(boolean reqCredDelegState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqCredDelegState = reqCredDelegState;
        }
    }
    
    @Override
    public void requestAnonymity(boolean reqAnonState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqAnonState = reqAnonState;
        }
    }
    
    @Override
    public void requestConf(boolean reqConfState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqConfState = reqConfState;
        }
    }
    
    @Override
    public void requestInteg(boolean reqIntegState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqIntegState = reqIntegState;
        }
    }
    
    @Override
    public void requestLifetime(int reqLifetime) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqLifetime = reqLifetime;
        }
    }
    
    @Override
    public void setChannelBinding(ChannelBinding channelBindings) throws GSSException {
        if (this.mechCtxt == null) {
            this.channelBindings = channelBindings;
        }
    }
    
    @Override
    public boolean getCredDelegState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getCredDelegState();
        }
        return this.reqCredDelegState;
    }
    
    @Override
    public boolean getMutualAuthState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getMutualAuthState();
        }
        return this.reqMutualAuthState;
    }
    
    @Override
    public boolean getReplayDetState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getReplayDetState();
        }
        return this.reqReplayDetState;
    }
    
    @Override
    public boolean getSequenceDetState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getSequenceDetState();
        }
        return this.reqSequenceDetState;
    }
    
    @Override
    public boolean getAnonymityState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getAnonymityState();
        }
        return this.reqAnonState;
    }
    
    @Override
    public boolean isTransferable() throws GSSException {
        return this.mechCtxt != null && this.mechCtxt.isTransferable();
    }
    
    @Override
    public boolean isProtReady() {
        return this.mechCtxt != null && this.mechCtxt.isProtReady();
    }
    
    @Override
    public boolean getConfState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getConfState();
        }
        return this.reqConfState;
    }
    
    @Override
    public boolean getIntegState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getIntegState();
        }
        return this.reqIntegState;
    }
    
    @Override
    public int getLifetime() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getLifetime();
        }
        return this.reqLifetime;
    }
    
    @Override
    public GSSName getSrcName() throws GSSException {
        if (this.srcName == null) {
            this.srcName = GSSNameImpl.wrapElement(this.gssManager, this.mechCtxt.getSrcName());
        }
        return this.srcName;
    }
    
    @Override
    public GSSName getTargName() throws GSSException {
        if (this.targName == null) {
            this.targName = GSSNameImpl.wrapElement(this.gssManager, this.mechCtxt.getTargName());
        }
        return this.targName;
    }
    
    @Override
    public Oid getMech() throws GSSException {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getMech();
        }
        return this.mechOid;
    }
    
    @Override
    public GSSCredential getDelegCred() throws GSSException {
        if (this.mechCtxt == null) {
            throw new GSSExceptionImpl(12, "No mechanism context yet!");
        }
        GSSCredentialSpi delegCred = this.mechCtxt.getDelegCred();
        return (delegCred == null) ? null : new GSSCredentialImpl(this.gssManager, delegCred);
    }
    
    @Override
    public boolean isInitiator() throws GSSException {
        return this.initiator;
    }
    
    @Override
    public void dispose() throws GSSException {
        this.currentState = 4;
        if (this.mechCtxt != null) {
            this.mechCtxt.dispose();
            this.mechCtxt = null;
        }
        this.myCred = null;
        this.srcName = null;
        this.targName = null;
    }
    
    @Override
    public Object inquireSecContext(InquireType inquireType) throws GSSException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new InquireSecContextPermission(inquireType.toString()));
        }
        if (this.mechCtxt == null) {
            throw new GSSException(12);
        }
        return this.mechCtxt.inquireSecContext(inquireType);
    }
    
    @Override
    public void requestDelegPolicy(boolean reqDelegPolicyState) throws GSSException {
        if (this.mechCtxt == null && this.initiator) {
            this.reqDelegPolicyState = reqDelegPolicyState;
        }
    }
    
    @Override
    public boolean getDelegPolicyState() {
        if (this.mechCtxt != null) {
            return this.mechCtxt.getDelegPolicyState();
        }
        return this.reqDelegPolicyState;
    }
}
